package com.microservices.zone.project.mapper;

import com.microservices.common.datascope.annotation.DataScope;
import com.microservices.zone.project.domain.ZoneProject;
import com.microservices.zone.project.domain.vo.ZoneProjectSearchVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/**
 * 特色专区项目聚合Mapper接口
 *
 * @author otto
 * @date 2023-05-11
 */
@Mapper
public interface ZoneProjectMapper {
    /**
     * 查询特色专区项目聚合
     *
     * @param id 特色专区项目聚合主键
     * @return 特色专区项目聚合
     */
    public ZoneProject selectZoneProjectById(Long id);

    /**
     * 查询特色专区项目聚合列表
     *
     * @param zoneProjectSearchVo 特色专区项目聚合
     * @param isOpen
     * @return 特色专区项目聚合集合
     */
    @DataScope(deptAlias = "zp")
    public List<ZoneProject> selectZoneProjectList(ZoneProjectSearchVo zoneProjectSearchVo);

    /**
     * 新增特色专区项目聚合
     *
     * @param zoneProject 特色专区项目聚合
     * @return 结果
     */
    public int insertZoneProject(ZoneProject zoneProject);

    /**
     * 修改特色专区项目聚合
     *
     * @param id
     * @param zoneProjectUpdateVo 特色专区项目聚合
     * @return 结果
     */
    public int updateZoneProject(ZoneProject zoneProject);

    /**
     * 删除特色专区项目聚合
     *
     * @param id 特色专区项目聚合主键
     * @return 结果
     */
    public int deleteZoneProjectById(Long id);

    /**
     * 批量删除特色专区项目聚合
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteZoneProjectByIds(Long[] ids);

    /**
     * 获取项目聚合数量
     *
     * @param zoneProjectSearchVo
     * @return
     */
    Integer selectZoneProjectListCount(ZoneProjectSearchVo zoneProjectSearchVo);

    /**
     * 根据GitLink项目Id及专区Id查找项目聚合
     *
     * @param gitlinkProjectId
     * @param zoneId
     * @return
     */
    ZoneProject selectZoneProjectByGitLinkProjectIdAndZoneId(@Param("gitlinkProjectId") Long gitlinkProjectId, @Param("zoneId") Long zoneId);

    /**
     * 更新项目名称和标识
     *
     * @param zoneProjectId
     * @param name
     * @param fullName
     */
    void updateZoneProjectNameAndFullName(@Param("zoneProjectId") Long zoneProjectId,
                                          @Param("name") String name,
                                          @Param("fullName") String fullName,
                                          @Param("updateTime") Date updateTime);

    /**
     * 查询专区下最大序号
     *
     * @param zoneId
     * @return
     */
    Integer selectMaxSortByZoneId(Long zoneId);

    /**
     * 查询专区下总数量
     *
     * @param zoneId
     * @return
     */
    Integer selectCountByZoneId(Long zoneId);

    /**
     * 获取指定序号的数据
     *
     * @param data
     * @param order
     * @param isContainsSelf
     * @return
     */
    ZoneProject selectDataByOrderAndZoneId(@Param("data") ZoneProject data, @Param("order") Integer order, @Param("isContainsSelf") boolean isContainsSelf);

    /**
     * 查询专区下最小排序值
     *
     * @param zoneId
     * @return
     */
    int selectMinSortByZoneId(Long zoneId);

    /**
     * 批量减少小于指定排序的排序值
     *
     * @param zoneId
     * @param setSort
     * @return
     */
    Integer batchUpdateSortDecrease(@Param("zoneId") Long zoneId, @Param("setSort") Integer setSort);

    /**
     * 获取专区下项目首页推荐数量
     *
     * @param zoneId
     * @return
     */
    Integer selectHomepageCount(Long zoneId);

    /**
     * 根据专区id删除专区项目
     *
     * @param zoneId
     * @return
     */
    int deleteZoneProjectByZoneId(Long zoneId);
}
